/**
 * @file dogm-cmd.c
 * libdogm
 *
 * If you need to transfer commands (cmd) to the lcd,
 * you have to use this functions.
 *
 * This module shares the cmd function for the dogm lcd display
 * http://www.lcd-module.de/eng/dog/dog.htm .
 *
 * Please note that this lib only has been tested with the dogm163.
 *
 * LICENSE:
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * @author JORDACIJEVIC Serz <serz@jordacijevic.name>
 * @date 2008/05/19
 *
 * @version $Revision: 1.0$
 *
 * @todo Nothing to do. - Everything perfect! ;-)
 *
 * URL: $HeadURL: http://serz.jordacijevic.name/?mmenu=project&smenu=libdogm$
 *
 * Last Modified: $Author: Jordacijevic Serz$
 */

/*
 * -------------------------------------------------------------- includes --
 */

#define _GNU_SOURCE

/* std */
#include <avr/io.h>

/* local */
#include "dogm-core.h"
#include "dogm-cmd.h"

/*
 * --------------------------------------------------------------- defines --
 */

/*
 * -------------------------------------------------------------- typedefs --
 */

#define LCD_DELAY_CLEAR		2
#define LCD_DELAY_POWER		45

#define LCD_DISP_SHIFT		0x01
#define LCD_AUTO_INC_LEFT	0x02

// Instruction Tabel
#define LCD_IS1			0x01
#define LCD_IS2			0x02

#define LCD_CLR_CMD		0x01
#define LCD_HOME_CMD		0x02
#define LCD_FSET_DH_CMD		0x04
#define LCD_DH_12_CMD		0x10
#define LCD_DH_21_CMD		0x18

// LCD Display Control
#define LCD_DISPLAY_CMD		0x08
#define LCD_BLINK_ON		0x01
#define LCD_CURSOR_ON		0x02
#define LCD_DISPLAY_ON		0x04

/*
 * --------------------------------------------------------------- globals --
 */

static void (*pdelay_ms)(double);

static uint8_t lcd_dim_x = 16;
static uint8_t lcd_dim_y = 3;

static char function_set = 0x38;	// default
static char entry_mode_set = 0x04;	// default

static void dogm_cmd_curser_home(void);

/*
 * ------------------------------------------------------------- functions --
 */

static void dogm_cmd_curser_home(void)
{
	dogm_core_cmd(LCD_HOME_CMD);	// Command
	pdelay_ms(LCD_DELAY_CLEAR);
}

/**
 *
 * \brief dogm_cmd_fset_is()
 *
 * selsect the instruction table.
 * IS[x,1]=(0x01), IS[2,x]=(0x02).
 *
 * Command		RS R/W	7654 3210	Hex	Note
 * Function Set		0 0	0011 1000	$38	IS[2,1]=[0,0]
 * Function Set		0 0	0011 1001	$39	IS[2,1]=[0,1]
 * Function Set		0 0	0011 1010	$3a	IS[2,1]=[1,0]
 *
 * \param is instruction table selection
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_fset_is(uint8_t is)
{
	if ((is & LCD_IS1) == LCD_IS1) {
		function_set |= LCD_IS1;	// IS 0 = HIGH
	} else {
		function_set &= ~LCD_IS1;	// IS 0 = LOW
	}

	if ((is & LCD_IS2) == LCD_IS2) {
		function_set |= LCD_IS2;	// IS 1 = HIGH
	} else {
		function_set &= ~LCD_IS2;	// IS 1 = LOW
	}

	dogm_core_cmd(function_set);
}

/**
 *
 * \brief dogm_cmd_fset_dh()
 *
 * is enables or disables the the double line module.
 *
 * \param pos enables (true) or disables (false)
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_fset_dh(uint8_t pos)
{
	if (pos) {
		function_set |= LCD_FSET_DH_CMD;	// set DH
	} else {
		function_set &= ~LCD_FSET_DH_CMD;	// remove DH
	}
	dogm_core_cmd(function_set);
}

/**
 *
 * \brief dogm_cmd_dh_pos()
 *
 * defines the position of the double line.
 * You can select the first or secund line.
 *
 * \param pos selects the first (false) or the secund line (true).
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_dh_pos(uint8_t pos)
{
	char fs = function_set;

	dogm_cmd_fset_is(2);		// set is=2
	if (pos) {
		dogm_core_cmd(LCD_DH_12_CMD);	// 1/2 lines
	} else {
		dogm_core_cmd(LCD_DH_21_CMD);	// 2/1 lines
	}
	function_set = fs;

	dogm_core_cmd(function_set);
}

/**
 *
 * \brief dogm_cmd_disp_clr()
 *
 * clears the display
 *
 * Command		RS R/W	7654 3210	Hex	Note
 * Clear Display	0 0	0000 0001	$01	delete display, cursor at home
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_disp_clr(void)
{
	dogm_core_cmd(LCD_CLR_CMD);	// Command
	pdelay_ms(LCD_DELAY_CLEAR);
}

/**
 *
 * \brief dogm_cmd_entry()
 *
 * selelect the display entry mode.
 *
 * Command		RS R/W	7654 3210	Hex	Note
 * Entry Mode Set	0 0	0000 0110	$06	cursor auto-increment
 *
 * \param entry shift shift of entire display is performed (0x01) and/or cursor decrement (0x02)
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_entry(uint8_t entry)
{
	if ((entry & LCD_DISP_SHIFT) == LCD_DISP_SHIFT) {
		// S = 1 -> shift of entire display is performed
		entry_mode_set |= LCD_DISP_SHIFT;
	} else {
		// S = 0 -> shift of entire display is NOT performed
		entry_mode_set &= ~LCD_DISP_SHIFT;
	}

	if ((entry & LCD_AUTO_INC_LEFT) == LCD_AUTO_INC_LEFT) {
		// I/D = "1" -> shift left
		entry_mode_set |= LCD_AUTO_INC_LEFT;
	} else {
		// I/D = "0" -> shift right
		entry_mode_set &= ~LCD_AUTO_INC_LEFT;
	}

	dogm_core_cmd(entry_mode_set);	// Command
}

/**
 *
 * \brief dogm_cmd_disp()
 *
 * controls the display.
 *
 * Command		RS R/W	7654 3210	Hex	Note
 * Display ON/OFF	0 0	0000 1111	$0f	display on, cursor on, cursor blink
 *
 * \param disp selects the display mode
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_disp(uint8_t disp)
{
	char cmd = LCD_DISPLAY_CMD;

	if ((disp & LCD_DISPLAY_ON) == LCD_DISPLAY_ON) {
		cmd |= LCD_DISPLAY_ON;	// display on
	}

	if ((disp & LCD_CURSOR_ON) == LCD_CURSOR_ON) {
		cmd |= LCD_CURSOR_ON;	// cursor on
	}

	if ((disp & LCD_BLINK_ON) == LCD_BLINK_ON) {
		cmd |= LCD_BLINK_ON;	// blink on
	}

	dogm_core_cmd(cmd);		// Command
}

/**
 *
 * \brief dogm_cmd_init()
 *
 * is initialing the display.
 *
 * \param lcd_x is the maximum col. of the lcd
 * \param lcd_y is the maximum row of the lcd
 * \param voltage is the schema has been used to connect the lcd. 5V (true) or 3V3 (false)
 * \param remore_delay_ms is the ref. to the _delay_ms function
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_init(uint8_t lcd_x, uint8_t lcd_y, uint8_t voltage, void (*remore_delay_ms)(double))
{
	lcd_dim_x = lcd_x;
	lcd_dim_y = lcd_y;
	pdelay_ms = remore_delay_ms;

	pdelay_ms(LCD_DELAY_POWER);	// after VDD stabil wait >40ms

	dogm_cmd_fset_is(1);		// IS 1
	
	dogm_cmd_fset_is(1);		// IS 1
           
	if (voltage) {			// 5V
// Bias Set		0 0	0001 1101	$1d	BS: 1/4, 3 line LCD
		dogm_core_cmd(0x1d);

// Power Control	0 0	0101 0000	$50	booster off, contrast C5, set C4
		dogm_core_cmd(0x50);

// Follower Control	0 0	0110 1100	$6c	set voltage follower and gain
		dogm_core_cmd(0x6c);

// Contrast Set		0 0	0111 1100	$7c	set contrast C3, C2, C1
		dogm_core_cmd(0x7c);

	} else {			// 3V3
// Bias Set		0 0	0001 0101	$15	BS: 1/5, 3 line LCD
		dogm_core_cmd(0x15);

// Power Control	0 0	0101 0101	$55	booster on, contrast C5, set C4
		dogm_core_cmd(0x55);

// Follower Control	0 0	0110 1110	$6e	set voltage follower and gain
		dogm_core_cmd(0x6e);

// Contrast Set		0 0	0111 0010	$72	set contrast C3, C2, C1
		dogm_core_cmd(0x72);
	}

	dogm_cmd_fset_is(0);	// IS 0

	dogm_cmd_disp(7);	// display on, cursor on, blink on

	dogm_cmd_disp_clr();	// Clear Display

	dogm_cmd_entry(2);	// cursor auto-increment

}

/**
 *
 * \brief dogm_cmd_curser_shift()
 *
 * shift the cursor
 *
 * \param dir left (false) or rlight (true)
 * \param count defines how many position to move
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_curser_shift(uint8_t dir, uint8_t count)
{
	char cmd = 0x10;		// shift left
	if (dir) {
		cmd |= 0x04;		// shift right
	}
	while (count--) {
		dogm_core_cmd(cmd);	// Command
	}
}

/**
 *
 * \brief dogm_cmd_disp_shift()
 *
 * shift the entire display
 *
 * \param dir left (false) or rlight (true)
 * \param count defines how many position to shift
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_disp_shift(uint8_t dir, uint8_t count)
{
	char cmd = 0x18;		// shift left
	if (dir) {
		cmd |= 0x04;		// shift right
	}
	while (count--) {
		dogm_core_cmd(cmd);	// Command
	}
}

/**
 *
 * \brief dogm_cmd_curser_set()
 *
 * set the cursor to a specific (x, y) position.
 *
 * \param x is the x position (col) on the display
 * \param y is the y position (row) on the display
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_cmd_curser_set(uint8_t x, uint8_t y)
{
	x %= lcd_dim_x;
	y %= lcd_dim_y;
	x += y * lcd_dim_x;
	dogm_cmd_curser_home();
	dogm_cmd_curser_shift(1, x);
}

