/**
 * @file dogm-data.c
 * libdogm
 *
 * If you need to transfer chars (data) to the lcd,
 * you have to use this functions.
 *
 * This module shares the data function for the dogm lcd display
 * http://www.lcd-module.de/eng/dog/dog.htm .
 *
 * Please note that this lib only has been tested with the dogm163.
 *
 * LICENSE:
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * @author JORDACIJEVIC Serz <serz@jordacijevic.name>
 * @date 2008/05/19
 *
 * @version $Revision: 1.0$
 *
 * @todo Nothing to do. - Everything perfect! ;-)
 *
 * URL: $HeadURL: http://serz.jordacijevic.name/?mmenu=project&smenu=libdogm$
 *
 * Last Modified: $Author: Jordacijevic Serz$
 */

/*
 * -------------------------------------------------------------- includes --
 */

#define _GNU_SOURCE

/* std */
#include <avr/io.h>
#include <math.h>

/* local */
#include "dogm-core.h"
#include "dogm-cmd.h"
#include "dogm-data.h"

/*
 * --------------------------------------------------------------- defines --
 */

#define LCD_INT_OFFSET		48

/*
 * -------------------------------------------------------------- typedefs --
 */

/*
 * --------------------------------------------------------------- globals --
 */

/*
 * ------------------------------------------------------------- functions --
 */

/**
 *
 * \brief dogm_data_char()
 *
 * send the data char to the lcd.
 *
 * \param c it the char you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_char(char c)
{
	dogm_core_data(c);
}

/**
 *
 * \brief dogm_data_char_xy()
 *
 * send the data char to the lcd.
 *
 * \param x is the x position (col) on the display
 * \param y is the y position (row) on the display
 * \param c it the char you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_char_xy(uint8_t x, uint8_t y, char c)
{
	dogm_cmd_curser_set(x, y);
	dogm_data_char(c);
}

/**
 *
 * \brief dogm_data_int()
 *
 * send the data int to the lcd.
 *
 * \param i it the int you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_int(int i)
{
	dogm_data_long((long) i);
}

/**
 *
 * \brief dogm_data_int_xy()
 *
 * send the data int to the lcd.
 *
 * \param x is the x position (col) on the display
 * \param y is the y position (row) on the display
 * \param i it the int you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_int_xy(uint8_t x, uint8_t y, int i)
{
	dogm_cmd_curser_set(x, y);
	dogm_data_int(i);
}

/**
 *
 * \brief dogm_data_uint()
 *
 * send the data unsigned int to the lcd.
 *
 * \param ui it the unsigned int you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_uint(unsigned int ui)
{
	dogm_data_ulong((unsigned long) ui);
}

/**
 *
 * \brief dogm_data_uint_xy()
 *
 * send the data unsigned int to the lcd.
 *
 * \param x is the x position (col) on the display
 * \param y is the y position (row) on the display
 * \param ui it the unsigned int you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_uint_xy(uint8_t x, uint8_t y, unsigned int ui)
{
	dogm_cmd_curser_set(x, y);
	dogm_data_uint(ui);
}

/**
 *
 * \brief dogm_data_long()
 *
 * send the data long to the lcd.
 *
 * \param l it the long you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_long(long l)
{
	if (l < 0) {
		dogm_data_char('-');
		dogm_data_ulong((unsigned long)(-l));
	} else {
		dogm_data_ulong((unsigned long)l);
	}
}

/**
 *
 * \brief dogm_data_long_xy()
 *
 * send the data long to the lcd.
 *
 * \param x is the x position (col) on the display
 * \param y is the y position (row) on the display
 * \param l it the long you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_long_xy(uint8_t x, uint8_t y, long l)
{
	dogm_cmd_curser_set(x, y);
	dogm_data_long(l);
}

/**
 *
 * \brief dogm_data_ulong()
 *
 * send the data unsigned long to the lcd.
 *
 * \param ul it the unsigned long you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_ulong(unsigned long ul)
{
	unsigned int len = 0;
	unsigned long tmp = 0;
	unsigned long val = 0;

	if (ul) {
		tmp = ul;
	} else {
		tmp = 1;	// force if zero
	}

	while (tmp) {
		len++;
		tmp /= 10;
	}

	while (len--) {
		val = ul;
		tmp = pow(10, len);
		if (tmp) { // no div NULL
			val /= tmp;
		}
		dogm_data_char((val % 10) + LCD_INT_OFFSET);
	}
}

/**
 *
 * \brief dogm_data_ulong_xy()
 *
 * send the data unsigned long to the lcd.
 *
 * \param x is the x position (col) on the display
 * \param y is the y position (row) on the display
 * \param ul it the unsigned long you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_ulong_xy(uint8_t x, uint8_t y, unsigned long ul)
{
	dogm_cmd_curser_set(x, y);
	dogm_data_ulong(ul);
}

/**
 *
 * \brief dogm_data_string()
 *
 * send the data char* to the lcd.
 *
 * \param s it the char* you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_string(char *s)
{
	while ((*s) != '\0') {
		dogm_data_char(*(s++));
	}
}

/**
 *
 * \brief dogm_data_string_xy()
 *
 * send the data char* to the lcd.
 *
 * \param x is the x position (col) on the display
 * \param y is the y position (row) on the display
 * \param s it the char* you want to send
 *
 * \return always void
 * \retval void always
 *
 */
void dogm_data_string_xy(uint8_t x, uint8_t y, char *s)
{
	dogm_cmd_curser_set(x, y);
	dogm_data_string(s);
}

